/*
 * hrcodec.c
 *
 * Simple HRLib COmpression/DECompression program.
 * Don`t forget to copy hr.dll into the working
 * directory before runing (if you compile with HR_WIN32_DLL).
 *  
 * (C) ENTROPYWARE, Alexander Simakov.
 *
 * Our site: http://www.entropyware.info/
 * Support:  support@entropyware.info
 * Author:   xander@entropyware.info
 */

#include <stdio.h>
#include <stdlib.h>
#include "pbm.h"
#include "haar.h"

void error(char *msg)
{
  printf("%s\n", msg);
  exit(1);
}

void decompress_file(char *in_path, char *out_path)
{
  FILE *in_file, *out_file;
  unsigned char *inp, *out;
  int buffer_size, width, height, res;

  /* open files */
  in_file = fopen(in_path, "rb");
  out_file = fopen(out_path, "wb");

  buffer_size = file_size(in_file);

  if (in_file == NULL || out_file == NULL) error("i/o error");

  /* allocate input buffer */
  inp = (unsigned char *) malloc(buffer_size);

  if (inp == NULL) error("not enough memory");

  /* load whole file into the memory */
  fread(inp, 1, buffer_size, in_file);

  /* get width and height from the header */
  hr_info(inp, &width, &height);

  /* allocate memory for image */
  out = (unsigned char *) malloc(width * height);

  /* decompression */
  res = hr_decompress(inp, buffer_size, out);

  /* something wrong */
  if (res) error("internal error");

  /* write PGM header */
  fprintf(out_file, "P5\n%d %d\n255\n", width, height);

  /* write decompressed image */
  fwrite(out, 1, width * height, out_file);
}

void compress_file(char *in_path, char *out_path, int quality)
{
  FILE *in_file, *out_file;
  unsigned char *inp, *out;
  int buffer_size, compressed_size, res;
  pbm_hdr h;

  /* open files */
  in_file = fopen(in_path, "rb");
  out_file = fopen(out_path, "wb");

  if (in_file == NULL || out_file == NULL) error("i/o error");

  /* read PGM file header */
  if ((read_hdr(in_file, &h) == -1) || (h.type != PGM)) error("invalid format");

  buffer_size = h.width * h.height + 1024;

  /* allocate memory for input/output buffers */
  inp = (unsigned char *) malloc(h.width * h.height);
  out = (unsigned char *) malloc(buffer_size);

  if (inp == NULL || out == NULL) error("not enough memory");

  /* load whole image into the memory */
  fread(inp, 1, h.width * h.height, in_file);

  /* compression */
  res = hr_compress(inp, h.width, h.height, quality, out, buffer_size, &compressed_size);

  /* something wrong */
  if (res) error("internal error");

  /* write compressed data */
  fwrite(out, 1, compressed_size, out_file);
}

int main(int argc, char *argv[])
{
  printf("\nHRCODEC, (C) ENTROPYWARE, Alexander Simakov.\n");

  if ((argc == 5) && (*argv[1] == 'c') && (atoi(argv[2]) >= 0) && (atoi(argv[2]) <= 100)) {
    compress_file(argv[3], argv[4], atoi(argv[2]));
    printf("\n");
    return 0;
  }

  if ((argc == 4) && (*argv[1] == 'd')) {
    decompress_file(argv[2], argv[3]);
    printf("\n");
    return 0;
  }

  printf("\nUsage: <hrcodec.exe> <c|d> [0<=quality<=100] <in_file> <out_file>\n\n");
  printf("Example 1: hrcodec.exe c 85 lena.pgm lena.hr\n");
  printf("Example 2: hrcodec.exe d goldhill.hr goldhill.pgm\n");

  return 1;
}
